@@ -10,13 +10,31 @@ from . import _pulsectl as c
 
 
 if sys.version_info.major >= 3:
-	long, unicode, print_err = int, str, ft.partial(print, file=sys.stderr, flush=True)
+	long, unicode = int, str
+	print_err = ft.partial(print, file=sys.stderr, flush=True)
+	def wrapper_with_sig_info(func, wrapper, index_arg=False):
+		sig = inspect.signature(func or (lambda: None))
+		if index_arg:
+			sig = sig.replace(parameters=[inspect.Parameter( 'index',
+				inspect.Parameter.POSITIONAL_OR_KEYWORD )] + list(sig.parameters.values()))
+		wrapper.__name__, wrapper.__signature__, wrapper.__doc__ = '', sig, func.__doc__
+		return wrapper
+
 else:
 	range, map = xrange, it.imap
 	def print_err(*args, **kws):
 		kws.setdefault('file', sys.stderr)
 		print(*args, **kws)
 		kws['file'].flush()
+	def wrapper_with_sig_info(func, wrapper, index_arg=False):
+		func_args = list(inspect.getargspec(func or (lambda: None)))
+		func_args[0] = list(func_args[0])
+		if index_arg: func_args[0] = ['index'] + func_args[0]
+		wrapper.__name__ = '...'
+		wrapper.__doc__ = 'Signature: func' + inspect.formatargspec(*func_args)
+		if func.__doc__: wrapper.__doc__ += '\n\n' + func.__doc__
+		return wrapper
+
 
 is_str = lambda v,ext=None,native=False: (
 	isinstance(v, ( (unicode, bytes)
@@ -62,14 +80,14 @@ class Enum(object):
 		self._name, self._values, self._c_vals = name, dict(), dict()
 		for c_val, k in vals:
 			v = EnumValue(name, k, c_val)
-			setattr(self, k, v)
+			setattr(self, k.replace('-', '_'), v)
 			self._c_vals[c_val] = self._values[k] = v
 
 	def __getitem__(self, k, *default):
 		if isinstance(k, EnumValue):
 			t, k, v = k._t, k._value, k
 			if t != self._name: raise KeyError(v)
-		try: return getattr(self, k, *default)
+		try: return getattr(self, k.replace('-', '_'), *default)
 		except AttributeError: raise KeyError(k)
 
 	def _get(self, k, default=None): return self.__getitem__(k, default)
@@ -139,6 +157,7 @@ class PulseObject(object):
 					None if not struct.active_port else cls_port(struct.active_port.contents) )
 			if hasattr(struct, 'channel_map'):
 				self.channel_count, self.channel_list = struct.channel_map.channels, list()
+				self.channel_list_raw = struct.channel_map.map[:self.channel_count]
 				if self.channel_count > 0:
 					s = c.create_string_buffer(b'\0' * 512)
 					c.pa.channel_map_snprint(s, len(s), struct.channel_map)
@@ -378,6 +397,13 @@ class Pulse(object):
 		self.event_masks = sorted(PulseEventMaskEnum._values.values())
 		self.event_callback = None
 
+		chan_names = dict()
+		for n in range(256):
+			name = c.pa.channel_position_to_string(n)
+			if name is None: break
+			chan_names[n] = name
+		self.channel_list_enum = Enum('channel_pos', chan_names)
+
 	def _ctx_init(self):
 		if self._ctx:
 			with self._loop_lock:
@@ -504,8 +530,8 @@ class Pulse(object):
 			ts = c.mono_time()
 			ts_deadline = timeout and (ts + timeout)
 			while True:
-				delay = max(0, int((ts_deadline - ts) * 1000)) if ts_deadline else -1
-				c.pa.mainloop_prepare(loop, delay) # delay in ms
+				delay = max(0, int((ts_deadline - ts) * 1000000)) if ts_deadline else -1
+				c.pa.mainloop_prepare(loop, delay) # delay in us
 				c.pa.mainloop_poll(loop)
 				if self._loop_closed: break # interrupted by close() or such
 				c.pa.mainloop_dispatch(loop)
@@ -541,10 +567,8 @@ class Pulse(object):
 				if not data: raise PulseIndexError(index)
 				data, = data
 			return data
-		_wrapper_method.__name__ = '...'
-		_wrapper_method.__doc__ = 'Signature: func({})'.format(
-			'' if pulse_func.__name__.endswith('_list') or singleton or not index_arg else 'index' )
-		return _wrapper_method
+		return wrapper_with_sig_info( None, _wrapper_method,
+			not (pulse_func.__name__.endswith('_list') or singleton or not index_arg) )
 
 	get_sink_by_name = _pulse_get_list(
 		c.PA_SINK_INFO_CB_T,
@@ -608,13 +632,7 @@ class Pulse(object):
 				try: pulse_op(self._ctx, *(list(pulse_args) + [cb, None]))
 				except c.ArgumentError as err: raise TypeError(err.args)
 				except c.pa.CallError as err: raise PulseOperationInvalid(err.args[-1])
-		func_args = list(inspect.getargspec(func or (lambda: None)))
-		func_args[0] = list(func_args[0])
-		if index_arg: func_args[0] = ['index'] + func_args[0]
-		_wrapper.__name__ = '...'
-		_wrapper.__doc__ = 'Signature: func' + inspect.formatargspec(*func_args)
-		if func.__doc__: _wrapper.__doc__ += '\n\n' + func.__doc__
-		return _wrapper
+		return wrapper_with_sig_info(func, _wrapper, index_arg)
 
 	card_profile_set_by_index = _pulse_method_call(
 		c.pa.context_set_card_profile_by_index, lambda profile_name: profile_name )
@@ -669,6 +687,8 @@ class Pulse(object):
 			try: c.pa.context_load_module(self._ctx, name, args, cb, None)
 			except c.pa.CallError as err: raise PulseOperationInvalid(err.args[-1])
 		index, = data
+		if index == c.PA_INVALID:
+			raise PulseError('Failed to load module: {} {}'.format(name, args))
 		return index
 
 	module_unload = _pulse_method_call(c.pa.context_unload_module, None)
@@ -802,6 +822,7 @@ class Pulse(object):
 	def event_callback_set(self, func):
 		'''Call event_listen() to start receiving these,
 				and be sure to raise PulseLoopStop in a callback to stop the loop.
+			Callback should accept single argument - PulseEventInfo instance.
 			Passing None will disable the thing.'''
 		self.event_callback = func
 
